@@ -45,9 +45,9 @@
 
 #include <kconfig.h>
 #include <QDebug>
-#include <klibrary.h>
 #include <klocalizedstring.h>
 #include <kdesktopfile.h>
+#include <kpluginloader.h> // to find kioslave modules
 #include <kprotocolmanager.h>
 #include <kprotocolinfo.h>
 #include <krun.h> // TODO port away from kiofilewidgets
@@ -137,7 +137,15 @@ KLauncher::KLauncher()
 
     mSlaveDebug = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_DEBUG_WAIT"));
     if (!mSlaveDebug.isEmpty()) {
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
         qWarning("Klauncher running in slave-debug mode for slaves of protocol '%s'", qPrintable(mSlaveDebug));
+#else
+        // Slave debug mode causes kdeinit to suspend the process waiting
+        // for the developer to attach gdb to the process; we do not have
+        // a good way of doing a similar thing if we are using QProcess.
+        mSlaveDebug.clear();
+        qWarning("slave-debug mode is not available as Klauncher is not using kdeinit");
+#endif
     }
     mSlaveValgrind = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_VALGRIND"));
     if (!mSlaveValgrind.isEmpty()) {
@@ -276,7 +284,11 @@ void KLauncher::processRequestReturn(int status, const QByteArray &requestData)
         // qDebug().nospace() << lastRequest->name << " (pid " << lastRequest->pid << ") up and running.";
         switch (lastRequest->dbus_startup_type) {
         case KService::DBusNone:
-            lastRequest->status = KLaunchRequest::Running;
+            if (lastRequest->wait) {
+                lastRequest->status = KLaunchRequest::Launching;
+            } else {
+                lastRequest->status = KLaunchRequest::Running;
+            }
             break;
         case KService::DBusUnique:
         case KService::DBusMulti:
@@ -318,6 +330,11 @@ KLauncher::processDied(pid_t pid, long exitStatus)
                 request->status = KLaunchRequest::Running;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
                 // qDebug() << pid << "running as a unique app";
+#endif
+            } else if(request->dbus_startup_type == KService::DBusNone && request->wait) {
+                request->status = KLaunchRequest::Running;
+#ifdef KLAUNCHER_VERBOSE_OUTPUT
+                // qDebug() << pid << "running as DBusNone with wait to true";
 #endif
             } else {
                 request->status = KLaunchRequest::Error;
@@ -625,6 +642,7 @@ void KLauncher::exec_blind(const QString &name, const QStringList &arg_list, con
     request->pid = 0;
     request->status = KLaunchRequest::Launching;
     request->envs = envs;
+    request->wait = false;
     // Find service, if any - strip path if needed
     KService::Ptr service = KService::serviceByDesktopName(name.mid(name.lastIndexOf(QLatin1Char('/')) + 1));
     if (service) {
@@ -769,6 +787,7 @@ KLauncher::start_service(KService::Ptr service, const QStringList &_urls,
 #endif
 
     request->pid = 0;
+    request->wait = false;
     request->envs = envs;
     send_service_startup_info(request, service, startup_id, envs);
 
@@ -886,13 +905,13 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
                         const QString &workdir, const QStringList &envs,
                         const QString &startup_id, bool wait, const QDBusMessage &msg)
 {
-    Q_UNUSED(wait)
     KLaunchRequest *request = new KLaunchRequest;
     request->autoStart = false;
     request->arg_list = args;
     request->name = app;
     request->dbus_startup_type = KService::DBusNone;
     request->pid = 0;
+    request->wait = wait;
 #if HAVE_X11
     request->startup_id = startup_id.toLocal8Bit();
 #endif
@@ -1015,19 +1034,25 @@ KLauncher::requestSlave(const QString &protocol,
         return slave->pid();
     }
 
-    QString name = KProtocolInfo::exec(protocol);
-    if (name.isEmpty()) {
+    QString slaveModule = KProtocolInfo::exec(protocol);
+    if (slaveModule.isEmpty()) {
         error = i18n("Unknown protocol '%1'.\n", protocol);
         return 0;
     }
+    KPluginLoader loader(slaveModule);
+    QString slaveModulePath = loader.fileName();
+    if (slaveModulePath.isEmpty()) {
+        error = i18n("Could not find the '%1' plugin.\n", slaveModule);
+        return 0;
+    }
 
     QStringList arg_list;
 #ifdef USE_KPROCESS_FOR_KIOSLAVES
-    arg_list << name;
+    arg_list << slaveModulePath;
     arg_list << protocol;
     arg_list << mConnectionServer.address().toString();
     arg_list << app_socket;
-    name = QFile::decodeName(CMAKE_INSTALL_PREFIX "/" LIBEXEC_INSTALL_DIR "/kioslave");
+    QString name = QFile::decodeName(CMAKE_INSTALL_PREFIX "/" KF5_LIBEXEC_INSTALL_DIR "/kioslave");
 #else
     QString arg1 = protocol;
     QString arg2 = mConnectionServer.address().toString();
@@ -1035,27 +1060,26 @@ KLauncher::requestSlave(const QString &protocol,
     arg_list.append(arg1);
     arg_list.append(arg2);
     arg_list.append(arg3);
+    QString name = slaveModulePath;
 #endif
 
     // qDebug() << "KLauncher: launching new slave " << name << " with protocol=" << protocol
     // << " args=" << arg_list << endl;
 
 #ifdef Q_OS_UNIX
-    if (mSlaveDebug == protocol) {
 #ifndef USE_KPROCESS_FOR_KIOSLAVES
+    // see comments where mSlaveDebug is set in KLauncher::KLauncher
+    if (mSlaveDebug == protocol) {
         klauncher_header request_header;
         request_header.cmd = LAUNCHER_DEBUG_WAIT;
         request_header.arg_length = 0;
         kde_safe_write(kdeinitSocket, &request_header, sizeof(request_header));
-#else
-        name = QString::fromLatin1("gdb");
-#endif
     }
+#endif
     if (mSlaveValgrind == protocol) {
+        arg_list.prepend(name);
 #ifndef USE_KPROCESS_FOR_KIOSLAVES // otherwise we've already done this
-        KLibrary lib(name);
-        arg_list.prepend(lib.fileName());
-        arg_list.prepend(QFile::decodeName(CMAKE_INSTALL_PREFIX "/" LIBEXEC_INSTALL_DIR "/kioslave"));
+        arg_list.prepend(QFile::decodeName(CMAKE_INSTALL_PREFIX "/" KF5_LIBEXEC_INSTALL_DIR "/kioslave"));
 #endif
         name = QString::fromLatin1("valgrind");
 
@@ -1072,6 +1096,7 @@ KLauncher::requestSlave(const QString &protocol,
     request->arg_list =  arg_list;
     request->dbus_startup_type = KService::DBusNone;
     request->pid = 0;
+    request->wait = false;
 #if HAVE_X11
     request->startup_id = "0"; // krazy:exclude=doublequote_chars
 #endif
